=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2011-12-05 14:12:23 +0000
+++ bzrlib/branch.py	2011-12-11 02:43:03 +0000
@@ -1607,21 +1607,6 @@
         return not (self == other)
 
     @classmethod
-    def find_format(klass, controldir, name=None):
-        """Return the format for the branch object in controldir."""
-        try:
-            transport = controldir.get_branch_transport(None, name=name)
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=name, bzrdir=controldir)
-        try:
-            format_string = transport.get_bytes("format")
-            return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NotBranchError(path=transport.base, bzrdir=controldir)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string, kind='branch')
-
-    @classmethod
     @deprecated_method(deprecated_in((2, 4, 0)))
     def get_default_format(klass):
         """Return the current default format."""
@@ -1664,10 +1649,6 @@
         """
         raise NotImplementedError(self.set_reference)
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short format description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -1800,7 +1781,7 @@
         """
         registry._LazyObjectGetter.__init__(self, module_name, member_name)
         self._format_string = format_string
-        
+
     def get_format_string(self):
         """See BranchFormat.get_format_string."""
         return self._format_string
@@ -2018,8 +1999,26 @@
             self.revision_id)
 
 
-class BranchFormatMetadir(BranchFormat):
-    """Common logic for meta-dir based branch formats."""
+class BranchFormatMetadir(bzrdir.BzrDirMetaComponentFormat, BranchFormat):
+    """Base class for branch formats that live in meta directories.
+    """
+
+    def __init__(self):
+        BranchFormat.__init__(self)
+        bzrdir.BzrDirMetaComponentFormat.__init__(self)
+
+    @classmethod
+    def find_format(klass, controldir, name=None):
+        """Return the format for the branch object in controldir."""
+        try:
+            transport = controldir.get_branch_transport(None, name=name)
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=name, bzrdir=controldir)
+        try:
+            format_string = transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NotBranchError(path=transport.base, bzrdir=controldir)
+        return klass._find_format(format_registry, 'branch', format_string)
 
     def _branch_class(self):
         """What class to instantiate on open calls."""
@@ -2062,18 +2061,11 @@
         self._run_post_branch_init_hooks(a_bzrdir, name, branch)
         return branch
 
-    def network_name(self):
-        """A simple byte string uniquely identifying this format for RPC calls.
-
-        Metadir branch formats use their format string.
-        """
-        return self.get_format_string()
-
     def open(self, a_bzrdir, name=None, _found=False, ignore_fallbacks=False,
             found_repository=None, possible_transports=None):
         """See BranchFormat.open()."""
         if not _found:
-            format = BranchFormat.find_format(a_bzrdir, name=name)
+            format = BranchFormatMetadir.find_format(a_bzrdir, name=name)
             if format.__class__ != self.__class__:
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))
@@ -2122,7 +2114,8 @@
     def _branch_class(self):
         return BzrBranch5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar-NG branch format 5\n"
 
@@ -2158,7 +2151,8 @@
     def _branch_class(self):
         return BzrBranch6
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 6 (bzr 0.15)\n"
 
@@ -2190,7 +2184,8 @@
     def _branch_class(self):
         return BzrBranch8
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 8 (needs bzr 1.15)\n"
 
@@ -2244,7 +2239,8 @@
     def _branch_class(self):
         return BzrBranch7
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar Branch Format 7 (needs bzr 1.6)\n"
 
@@ -2276,7 +2272,8 @@
      - a format string
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BranchFormat.get_format_string()."""
         return "Bazaar-NG Branch Reference Format 1\n"
 
@@ -2342,7 +2339,7 @@
         :param possible_transports: An optional reusable transports list.
         """
         if not _found:
-            format = BranchFormat.find_format(a_bzrdir, name=name)
+            format = BranchFormatMetadir.find_format(a_bzrdir, name=name)
             if format.__class__ != self.__class__:
                 raise AssertionError("wrong format %r found for %r" %
                     (format, self))

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2011-12-05 14:12:23 +0000
+++ bzrlib/bzrdir.py	2011-12-11 03:44:06 +0000
@@ -850,8 +850,8 @@
 
         This might be a synthetic object for e.g. RemoteBranch and SVN.
         """
-        from bzrlib.branch import BranchFormat
-        return BranchFormat.find_format(self, name=name)
+        from bzrlib.branch import BranchFormatMetadir
+        return BranchFormatMetadir.find_format(self, name=name)
 
     def _get_mkdir_mode(self):
         """Figure out the mode to use when creating a bzrdir subdir."""
@@ -861,8 +861,8 @@
 
     def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
-        from bzrlib.branch import BranchFormat
-        format = BranchFormat.find_format(self, name=name)
+        from bzrlib.branch import BranchFormatMetadir
+        format = BranchFormatMetadir.find_format(self, name=name)
         return format.get_reference(self, name=name)
 
     def get_branch_transport(self, branch_format, name=None):
@@ -917,9 +917,9 @@
         Note: if you're going to open the working tree, you should just go
         ahead and try, and not ask permission first.
         """
-        from bzrlib.workingtree import WorkingTreeFormat
+        from bzrlib.workingtree import WorkingTreeFormatMetaDir
         try:
-            WorkingTreeFormat.find_format_string(self)
+            WorkingTreeFormatMetaDir.find_format_string(self)
         except errors.NoWorkingTree:
             return False
         return True
@@ -966,16 +966,16 @@
 
     def open_repository(self, unsupported=False):
         """See BzrDir.open_repository."""
-        from bzrlib.repository import RepositoryFormat
-        format = RepositoryFormat.find_format(self)
+        from bzrlib.repository import RepositoryFormatMetaDir
+        format = RepositoryFormatMetaDir.find_format(self)
         format.check_support_status(unsupported)
         return format.open(self, _found=True)
 
     def open_workingtree(self, unsupported=False,
             recommend_upgrade=True):
         """See BzrDir.open_workingtree."""
-        from bzrlib.workingtree import WorkingTreeFormat
-        format = WorkingTreeFormat.find_format(self)
+        from bzrlib.workingtree import WorkingTreeFormatMetaDir
+        format = WorkingTreeFormatMetaDir.find_format(self)
         format.check_support_status(unsupported, recommend_upgrade,
             basedir=self.root_transport.base)
         return format.open(self, _found=True)
@@ -1102,6 +1102,49 @@
         return self.transport.clone(path)
 
 
+class BzrDirMetaComponentFormat(controldir.ControlComponentFormat):
+    """Base class for all formats of things living in metadirs.
+
+    This class manages the format string that is stored in the 'format'
+    or 'branch-format' file.
+
+    All classes for (branch-, repository-, workingtree-) formats that
+    live in meta directories and have their own 'format' file
+    (i.e. different from .bzr/branch-format) derive from this class,
+    as well as the relevant base class for their kind
+    (BranchFormat, WorkingTreeFormat, RepositoryFormat).
+    """
+
+    @classmethod
+    def get_format_string(cls):
+        """Return the ASCII format string that identifies this format."""
+        raise NotImplementedError(cls.get_format_string)
+
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise ValueError("Invalid format header %r" % format_string)
+        return cls()
+
+    @classmethod
+    def _find_format(klass, registry, kind, format_string):
+        try:
+            cls = registry.get(format_string)
+        except KeyError:
+            raise errors.UnknownFormatError(format=format_string, kind=kind)
+        return cls
+
+    def network_name(self):
+        """A simple byte string uniquely identifying this format for RPC calls.
+
+        Metadir branch formats use their format string.
+        """
+        return self.get_format_string()
+
+    def __eq__(self, other):
+        return (self.__class__ is other.__class__)
+
+
 class BzrProber(controldir.Prober):
     """Prober for formats that use a .bzr/ control directory."""
 
@@ -1126,9 +1169,10 @@
         except errors.NoSuchFile:
             raise errors.NotBranchError(path=transport.base)
         try:
-            return klass.formats.get(format_string)
+            cls = klass.formats.get(format_string)
         except KeyError:
             raise errors.UnknownFormatError(format=format_string, kind='bzrdir')
+        return cls.from_string(format_string)
 
     @classmethod
     def known_formats(cls):
@@ -1557,6 +1601,12 @@
         """See BzrDirFormat.get_format_description()."""
         return "Meta directory format 1"
 
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise ValueError("Invalid format string %r" % format_string)
+        return cls()
+
     def network_name(self):
         return self.get_format_string()
 
@@ -1612,6 +1662,9 @@
     def __set_workingtree_format(self, wt_format):
         self._workingtree_format = wt_format
 
+    def __repr__(self):
+        return "<%r>" % (self.__class__.__name__,)
+
     workingtree_format = property(__get_workingtree_format,
                                   __set_workingtree_format)
 

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2011-11-27 16:55:11 +0000
+++ bzrlib/controldir.py	2011-12-07 14:49:51 +0000
@@ -830,10 +830,6 @@
 
     upgrade_recommended = False
 
-    def get_format_string(self):
-        """Return the format of this format, if usable in meta directories."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -866,6 +862,10 @@
             ui.ui_factory.recommend_upgrade(
                 self.get_format_description(), basedir)
 
+    @classmethod
+    def get_format_string(cls):
+        raise NotImplementedError(cls.get_format_string)
+
 
 class ControlComponentFormatRegistry(registry.FormatRegistry):
     """A registry for control components (branch, workingtree, repository)."""

=== modified file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	2011-11-27 17:34:50 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	2011-12-07 14:03:01 +0000
@@ -81,6 +81,12 @@
             stack_on_pwd=stack_on_pwd, repo_format_name=repo_format_name,
             make_working_trees=make_working_trees, shared_repo=shared_repo)
 
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise AssertionError("unexpected format string %r" % format_string)
+        return cls()
+
 
 class BzrDirFormat5(BzrDirFormatAllInOne):
     """Bzr control format 5.
@@ -98,7 +104,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 5\n"
 
@@ -161,7 +168,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 6\n"
 
@@ -669,7 +677,8 @@
     def __eq__(self, other):
         return type(self) == type(other)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrDirFormat.get_format_string()."""
         return "Bazaar-NG branch, format 0.0.4\n"
 
@@ -708,6 +717,12 @@
         return RepositoryFormat4()
     repository_format = property(__return_repository_format)
 
+    @classmethod
+    def from_string(cls, format_string):
+        if format_string != cls.get_format_string():
+            raise AssertionError("unexpected format string %r" % format_string)
+        return cls()
+
 
 class BzrDirPreSplitOut(BzrDir):
     """A common class for the all-in-one formats."""

=== modified file 'bzrlib/plugins/weave_fmt/repository.py'
--- bzrlib/plugins/weave_fmt/repository.py	2011-09-22 00:44:49 +0000
+++ bzrlib/plugins/weave_fmt/repository.py	2011-12-07 14:03:01 +0000
@@ -52,7 +52,7 @@
 from bzrlib.decorators import needs_read_lock, needs_write_lock
 from bzrlib.repository import (
     InterRepository,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     )
 from bzrlib.store.text import TextStore
 from bzrlib.versionedfile import (
@@ -504,7 +504,8 @@
     def _serializer(self):
         return xml5.serializer_v5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Repository format 7"
 
@@ -561,7 +562,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2011-12-03 03:30:10 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2011-12-07 14:03:01 +0000
@@ -1388,7 +1388,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar repository format 2a (needs bzr 1.16 or later)\n')
 
     def get_format_description(self):
@@ -1410,7 +1411,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return ('Bazaar development format 8\n')
 
     def get_format_description(self):

=== modified file 'bzrlib/repofmt/knitpack_repo.py'
--- bzrlib/repofmt/knitpack_repo.py	2011-05-02 15:29:58 +0000
+++ bzrlib/repofmt/knitpack_repo.py	2011-12-07 14:03:01 +0000
@@ -160,7 +160,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar pack repository format 1 (needs bzr 0.92)\n"
 
@@ -200,7 +201,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n"
 
@@ -238,7 +240,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return ("Bazaar pack repository format 1 with rich root"
                 " (needs bzr 1.0)\n")
@@ -276,7 +279,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n"
 
@@ -316,7 +320,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n"
 
@@ -362,7 +367,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n"
 
@@ -400,7 +406,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n"
 
@@ -437,7 +444,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n"
 
@@ -477,7 +485,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return ("Bazaar development format 2 with subtree support "
             "(needs bzr.dev from before 1.8)\n")

=== modified file 'bzrlib/repofmt/knitrepo.py'
--- bzrlib/repofmt/knitrepo.py	2011-06-16 14:54:47 +0000
+++ bzrlib/repofmt/knitrepo.py	2011-12-07 14:03:01 +0000
@@ -38,7 +38,7 @@
 from bzrlib.repository import (
     InterRepository,
     IsInWriteGroupError,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     )
 from bzrlib.vf_repository import (
     InterSameDataRepository,
@@ -321,7 +321,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:
@@ -367,7 +367,8 @@
     def __ne__(self, other):
         return self.__class__ is not other.__class__
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar-NG Knit Repository Format 1"
 
@@ -409,7 +410,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Bazaar Knit Repository Format 3 (bzr 0.15)\n"
 
@@ -450,7 +452,8 @@
 
     _matchingbzrdir = property(_get_matching_bzrdir, _ignore_setting_bzrdir)
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return 'Bazaar Knit Repository Format 4 (bzr 1.0)\n'
 

=== modified file 'bzrlib/repofmt/pack_repo.py'
--- bzrlib/repofmt/pack_repo.py	2011-07-25 07:51:35 +0000
+++ bzrlib/repofmt/pack_repo.py	2011-12-07 14:03:01 +0000
@@ -55,7 +55,7 @@
 from bzrlib.repository import (
     _LazyListJoin,
     MetaDirRepository,
-    RepositoryFormat,
+    RepositoryFormatMetaDir,
     RepositoryWriteLockResult,
     )
 from bzrlib.vf_repository import (
@@ -1906,7 +1906,7 @@
                                     than normal. I.e. during 'upgrade'.
         """
         if not _found:
-            format = RepositoryFormat.find_format(a_bzrdir)
+            format = RepositoryFormatMetaDir.find_format(a_bzrdir)
         if _override_transport is not None:
             repo_transport = _override_transport
         else:

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2011-12-05 14:12:23 +0000
+++ bzrlib/repository.py	2011-12-07 14:03:01 +0000
@@ -1409,24 +1409,6 @@
         return not self == other
 
     @classmethod
-    def find_format(klass, a_bzrdir):
-        """Return the format for the repository object in a_bzrdir.
-
-        This is used by bzr native formats that have a "format" file in
-        the repository.  Other methods may be used by different types of
-        control directory.
-        """
-        try:
-            transport = a_bzrdir.get_repository_transport(None)
-            format_string = transport.get_bytes("format")
-            return format_registry.get(format_string)
-        except errors.NoSuchFile:
-            raise errors.NoRepositoryPresent(a_bzrdir)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string,
-                                            kind='repository')
-
-    @classmethod
     @symbol_versioning.deprecated_method(symbol_versioning.deprecated_in((2, 4, 0)))
     def register_format(klass, format):
         format_registry.register(format)
@@ -1442,14 +1424,6 @@
         """Return the current default format."""
         return format_registry.get_default()
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format.
-
-        Note that in pre format ?? repositories the format string is
-        not permitted nor written to disk.
-        """
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -1521,7 +1495,7 @@
             hook(params)
 
 
-class MetaDirRepositoryFormat(RepositoryFormat):
+class RepositoryFormatMetaDir(bzrdir.BzrDirMetaComponentFormat, RepositoryFormat):
     """Common base class for the new repositories using the metadir layout."""
 
     rich_root_data = False
@@ -1537,7 +1511,8 @@
         return matching
 
     def __init__(self):
-        super(MetaDirRepositoryFormat, self).__init__()
+        RepositoryFormat.__init__(self)
+        bzrdir.BzrDirMetaComponentFormat.__init__(self)
 
     def _create_control_files(self, a_bzrdir):
         """Create the required files and the initial control_files object."""
@@ -1567,9 +1542,20 @@
         finally:
             control_files.unlock()
 
-    def network_name(self):
-        """Metadir formats have matching disk and network format strings."""
-        return self.get_format_string()
+    @classmethod
+    def find_format(klass, a_bzrdir):
+        """Return the format for the repository object in a_bzrdir.
+
+        This is used by bzr native formats that have a "format" file in
+        the repository.  Other methods may be used by different types of
+        control directory.
+        """
+        try:
+            transport = a_bzrdir.get_repository_transport(None)
+            format_string = transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoRepositoryPresent(a_bzrdir)
+        return klass._find_format(format_registry, 'repository', format_string)
 
 
 # formats which have no format string are not discoverable or independently

=== modified file 'bzrlib/tests/blackbox/test_upgrade.py'
--- bzrlib/tests/blackbox/test_upgrade.py	2011-10-23 13:56:11 +0000
+++ bzrlib/tests/blackbox/test_upgrade.py	2011-12-07 14:03:01 +0000
@@ -59,7 +59,8 @@
     def get_converter(self, format=None):
         return ConvertOldTestToMeta()
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Ancient Test Format"
 
     def _open(self, transport):

=== modified file 'bzrlib/tests/per_repository/test_repository.py'
--- bzrlib/tests/per_repository/test_repository.py	2011-11-28 11:44:21 +0000
+++ bzrlib/tests/per_repository/test_repository.py	2011-12-07 14:03:01 +0000
@@ -179,7 +179,7 @@
         except NotImplementedError:
             return
         self.assertEqual(self.repository_format,
-                         repository.RepositoryFormat.find_format(opened_control))
+             repository.RepositoryFormatMetaDir.find_format(opened_control))
 
     def test_format_matchingbzrdir(self):
         self.assertEqual(self.repository_format,

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2011-11-27 17:42:25 +0000
+++ bzrlib/tests/test_branch.py	2011-12-11 03:39:32 +0000
@@ -101,14 +101,15 @@
     # recursive section - that is, it appends the branch name.
 
 
-class SampleBranchFormat(_mod_branch.BranchFormat):
+class SampleBranchFormat(_mod_branch.BranchFormatMetadir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrBranchFormat.get_format_string()."""
         return "Sample branch format."
 
@@ -132,10 +133,11 @@
 SampleSupportedBranchFormatString = "Sample supported branch format."
 
 # And the format class can then reference the constant to avoid skew.
-class SampleSupportedBranchFormat(_mod_branch.BranchFormat):
+class SampleSupportedBranchFormat(_mod_branch.BranchFormatMetadir):
     """A sample supported format."""
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See BzrBranchFormat.get_format_string()."""
         return SampleSupportedBranchFormatString
 
@@ -180,7 +182,7 @@
             dir = format._matchingbzrdir.initialize(url)
             dir.create_repository()
             format.initialize(dir)
-            found_format = _mod_branch.BranchFormat.find_format(dir)
+            found_format = _mod_branch.BranchFormatMetadir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(_mod_branch.BzrBranchFormat5(), "bar")
 
@@ -195,17 +197,24 @@
         b = _mod_branch.Branch.open(self.get_url())
         self.assertEqual(b, "opened supported branch.")
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleBranchFormat.from_string("Sample branch format."),
+            SampleBranchFormat)
+        self.assertRaises(ValueError,
+            SampleBranchFormat.from_string, "Different branch format.")
+
     def test_find_format_not_branch(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(errors.NotBranchError,
-                          _mod_branch.BranchFormat.find_format,
+                          _mod_branch.BranchFormatMetadir.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleBranchFormat().initialize(dir)
         self.assertRaises(errors.UnknownFormatError,
-                          _mod_branch.BranchFormat.find_format,
+                          _mod_branch.BranchFormatMetadir.find_format,
                           dir)
 
     def test_register_unregister_format(self):
@@ -716,6 +725,7 @@
         f = StringIO()
         r.report(f)
         self.assertEqual("Now on revision 20.\n", f.getvalue())
+        self.assertEqual("Now on revision 20.\n", f.getvalue())
 
     def test_report_unchanged(self):
         r = _mod_branch.PullResult()

=== modified file 'bzrlib/tests/test_bzrdir.py'
--- bzrlib/tests/test_bzrdir.py	2011-11-27 17:42:25 +0000
+++ bzrlib/tests/test_bzrdir.py	2011-12-07 14:03:01 +0000
@@ -253,6 +253,10 @@
     def open(self, transport, _found=None):
         return "opened branch."
 
+    @classmethod
+    def from_string(cls, format_string):
+        return cls()
+
 
 class BzrDirFormatTest1(bzrdir.BzrDirMetaFormat1):
 

=== modified file 'bzrlib/tests/test_foreign.py'
--- bzrlib/tests/test_foreign.py	2011-11-27 17:42:25 +0000
+++ bzrlib/tests/test_foreign.py	2011-12-07 14:03:01 +0000
@@ -145,7 +145,8 @@
     repository_class = DummyForeignVcsRepository
     _commit_builder_class = DummyForeignCommitBuilder
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Dummy Foreign Vcs Repository"
 
     def get_format_description(self):
@@ -229,7 +230,8 @@
 
 class DummyForeignVcsBranchFormat(branch.BzrBranchFormat6):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Branch for Testing"
 
     @property

=== modified file 'bzrlib/tests/test_repository.py'
--- bzrlib/tests/test_repository.py	2011-12-05 14:12:23 +0000
+++ bzrlib/tests/test_repository.py	2011-12-11 03:42:09 +0000
@@ -92,14 +92,15 @@
                               old_format.__class__)
 
 
-class SampleRepositoryFormat(repository.RepositoryFormat):
+class SampleRepositoryFormat(repository.RepositoryFormatMetaDir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open(unsupported=True) routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See RepositoryFormat.get_format_string()."""
         return "Sample .bzr repository format."
 
@@ -137,21 +138,30 @@
             dir = format._matchingbzrdir.initialize(url)
             format.initialize(dir)
             t = transport.get_transport_from_path(url)
-            found_format = repository.RepositoryFormat.find_format(dir)
+            found_format = repository.RepositoryFormatMetaDir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(repository.format_registry.get_default(), "bar")
 
     def test_find_format_no_repository(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         self.assertRaises(errors.NoRepositoryPresent,
-                          repository.RepositoryFormat.find_format,
+                          repository.RepositoryFormatMetaDir.find_format,
                           dir)
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleRepositoryFormat.from_string(
+                "Sample .bzr repository format."),
+            SampleRepositoryFormat)
+        self.assertRaises(ValueError,
+            SampleRepositoryFormat.from_string,
+                "Different .bzr repository format.")
+
     def test_find_format_unknown_format(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize(self.get_url())
         SampleRepositoryFormat().initialize(dir)
         self.assertRaises(UnknownFormatError,
-                          repository.RepositoryFormat.find_format,
+                          repository.RepositoryFormatMetaDir.find_format,
                           dir)
 
     def test_register_unregister_format(self):
@@ -438,13 +448,15 @@
 
 class TestRepositoryFormat1(knitrepo.RepositoryFormatKnit1):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Test Format 1"
 
 
 class TestRepositoryFormat2(knitrepo.RepositoryFormatKnit1):
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         return "Test Format 2"
 
 

=== modified file 'bzrlib/tests/test_workingtree.py'
--- bzrlib/tests/test_workingtree.py	2011-05-13 12:51:05 +0000
+++ bzrlib/tests/test_workingtree.py	2011-12-11 02:43:03 +0000
@@ -79,6 +79,13 @@
             workingtree.format_registry.set_default(old_format)
         self.assertEqual(old_format, workingtree.format_registry.get_default())
 
+    def test_from_string(self):
+        self.assertIsInstance(
+            SampleTreeFormat.from_string("Sample tree format."),
+            SampleTreeFormat)
+        self.assertRaises(ValueError,
+            SampleTreeFormat.from_string, "Different format string.")
+
     def test_get_set_default_format_by_key(self):
         old_format = workingtree.format_registry.get_default()
         # default is 6
@@ -120,14 +127,15 @@
         self.assertEqual('subdir', relpath)
 
 
-class SampleTreeFormat(workingtree.WorkingTreeFormat):
+class SampleTreeFormat(workingtree.WorkingTreeFormatMetaDir):
     """A sample format
 
     this format is initializable, unsupported to aid in testing the
     open and open_downlevel routines.
     """
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Sample tree format."
 
@@ -172,14 +180,14 @@
         # is the right format object found for a working tree?
         branch = self.make_branch('branch')
         self.assertRaises(errors.NoWorkingTree,
-            workingtree.WorkingTreeFormat.find_format_string, branch.bzrdir)
+            workingtree.WorkingTreeFormatMetaDir.find_format_string, branch.bzrdir)
         transport = branch.bzrdir.get_workingtree_transport(None)
         transport.mkdir('.')
         transport.put_bytes("format", "some format name")
         # The format does not have to be known by Bazaar,
         # find_format_string just retrieves the name
         self.assertEquals("some format name",
-            workingtree.WorkingTreeFormat.find_format_string(branch.bzrdir))
+            workingtree.WorkingTreeFormatMetaDir.find_format_string(branch.bzrdir))
 
     def test_find_format(self):
         # is the right format object found for a working tree?
@@ -191,14 +199,14 @@
             dir.create_branch()
             format.initialize(dir)
             t = transport.get_transport(url)
-            found_format = workingtree.WorkingTreeFormat.find_format(dir)
+            found_format = workingtree.WorkingTreeFormatMetaDir.find_format(dir)
             self.assertIsInstance(found_format, format.__class__)
         check_format(workingtree_3.WorkingTreeFormat3(), "bar")
 
     def test_find_format_no_tree(self):
         dir = bzrdir.BzrDirMetaFormat1().initialize('.')
         self.assertRaises(errors.NoWorkingTree,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_find_format_unknown_format(self):
@@ -207,7 +215,7 @@
         dir.create_branch()
         SampleTreeFormat().initialize(dir)
         self.assertRaises(errors.UnknownFormatError,
-                          workingtree.WorkingTreeFormat.find_format,
+                          workingtree.WorkingTreeFormatMetaDir.find_format,
                           dir)
 
     def test_register_unregister_format(self):

=== modified file 'bzrlib/vf_repository.py'
--- bzrlib/vf_repository.py	2011-12-06 16:05:01 +0000
+++ bzrlib/vf_repository.py	2011-12-07 14:03:01 +0000
@@ -66,7 +66,7 @@
     CommitBuilder,
     InterRepository,
     MetaDirRepository,
-    MetaDirRepositoryFormat,
+    RepositoryFormatMetaDir,
     Repository,
     RepositoryFormat,
     )
@@ -2006,7 +2006,7 @@
             control_files)
 
 
-class MetaDirVersionedFileRepositoryFormat(MetaDirRepositoryFormat,
+class MetaDirVersionedFileRepositoryFormat(RepositoryFormatMetaDir,
         VersionedFileRepositoryFormat):
     """Base class for repository formats using versioned files in metadirs."""
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2011-11-28 15:18:59 +0000
+++ bzrlib/workingtree.py	2011-12-07 14:03:01 +0000
@@ -3029,25 +3029,6 @@
 
     supports_versioned_directories = None
 
-    @classmethod
-    def find_format_string(klass, controldir):
-        """Return format name for the working tree object in controldir."""
-        try:
-            transport = controldir.get_workingtree_transport(None)
-            return transport.get_bytes("format")
-        except errors.NoSuchFile:
-            raise errors.NoWorkingTree(base=transport.base)
-
-    @classmethod
-    def find_format(klass, controldir):
-        """Return the format for the working tree object in controldir."""
-        try:
-            format_string = klass.find_format_string(controldir)
-            return format_registry.get(format_string)
-        except KeyError:
-            raise errors.UnknownFormatError(format=format_string,
-                                            kind="working tree")
-
     def initialize(self, controldir, revision_id=None, from_branch=None,
                    accelerator_tree=None, hardlink=False):
         """Initialize a new working tree in controldir.
@@ -3078,10 +3059,6 @@
         """Return the current default format."""
         return format_registry.get_default()
 
-    def get_format_string(self):
-        """Return the ASCII format string that identifies this format."""
-        raise NotImplementedError(self.get_format_string)
-
     def get_format_description(self):
         """Return the short description for this format."""
         raise NotImplementedError(self.get_format_description)
@@ -3148,6 +3125,30 @@
         return self._matchingbzrdir
 
 
+class WorkingTreeFormatMetaDir(bzrdir.BzrDirMetaComponentFormat, WorkingTreeFormat):
+    """Base class for working trees that live in bzr meta directories."""
+
+    def __init__(self):
+        WorkingTreeFormat.__init__(self)
+        bzrdir.BzrDirMetaComponentFormat.__init__(self)
+
+    @classmethod
+    def find_format_string(klass, controldir):
+        """Return format name for the working tree object in controldir."""
+        try:
+            transport = controldir.get_workingtree_transport(None)
+            return transport.get_bytes("format")
+        except errors.NoSuchFile:
+            raise errors.NoWorkingTree(base=transport.base)
+
+    @classmethod
+    def find_format(klass, controldir):
+        """Return the format for the working tree object in controldir."""
+        format_string = klass.find_format_string(controldir)
+        return klass._find_format(format_registry, 'working tree',
+                format_string)
+
+
 format_registry.register_lazy("Bazaar Working Tree Format 4 (bzr 0.15)\n",
     "bzrlib.workingtree_4", "WorkingTreeFormat4")
 format_registry.register_lazy("Bazaar Working Tree Format 5 (bzr 1.11)\n",

=== modified file 'bzrlib/workingtree_3.py'
--- bzrlib/workingtree_3.py	2011-11-28 15:18:59 +0000
+++ bzrlib/workingtree_3.py	2011-12-07 14:03:01 +0000
@@ -37,7 +37,7 @@
 from bzrlib.transport.local import LocalTransport
 from bzrlib.workingtree import (
     InventoryWorkingTree,
-    WorkingTreeFormat,
+    WorkingTreeFormatMetaDir,
     )
 
 
@@ -136,7 +136,7 @@
             self.branch.unlock()
 
 
-class WorkingTreeFormat3(WorkingTreeFormat):
+class WorkingTreeFormat3(WorkingTreeFormatMetaDir):
     """The second working tree format updated to record a format marker.
 
     This format:
@@ -154,7 +154,8 @@
 
     supports_versioned_directories = True
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar-NG Working Tree format 3"
 
@@ -259,6 +260,3 @@
                                 _format=self,
                                 _bzrdir=a_bzrdir,
                                 _control_files=control_files)
-
-    def __str__(self):
-        return self.get_format_string()

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2011-12-05 17:59:48 +0000
+++ bzrlib/workingtree_4.py	2011-12-07 14:03:01 +0000
@@ -71,7 +71,7 @@
 from bzrlib.workingtree import (
     InventoryWorkingTree,
     WorkingTree,
-    WorkingTreeFormat,
+    WorkingTreeFormatMetaDir,
     )
 
 
@@ -1448,7 +1448,7 @@
         return views.PathBasedViews(self)
 
 
-class DirStateWorkingTreeFormat(WorkingTreeFormat):
+class DirStateWorkingTreeFormat(WorkingTreeFormatMetaDir):
 
     missing_parent_conflicts = True
 
@@ -1620,7 +1620,8 @@
 
     _tree_class = WorkingTree4
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 4 (bzr 0.15)\n"
 
@@ -1637,7 +1638,8 @@
 
     _tree_class = WorkingTree5
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 5 (bzr 1.11)\n"
 
@@ -1657,7 +1659,8 @@
 
     _tree_class = WorkingTree6
 
-    def get_format_string(self):
+    @classmethod
+    def get_format_string(cls):
         """See WorkingTreeFormat.get_format_string()."""
         return "Bazaar Working Tree Format 6 (bzr 1.14)\n"
 

